@@ -1,4 +1,5 @@
 package Lingua::YaTeA::Message;
+
 use strict;
 use warnings;
 
@@ -34,27 +35,40 @@ __END__
 
 =head1 NAME
 
-Lingua::YaTeA::Message - Perl extension for ???
+Lingua::YaTeA::Message - Perl extension for managing a message in the term extractor
 
 =head1 SYNOPSIS
 
   use Lingua::YaTeA::Message;
-  Lingua::YaTeA::Message->();
+  Lingua::YaTeA::Message->new($name, $content, $language);
 
 =head1 DESCRIPTION
 
+This module manages the message used in the interface. As the language
+of the interface can be parametrized, the message can have several
+contents. Each message is dedicated to a specific event in the term
+extractor. Messages are defined in the file
+C<share/YaTeA/locale/FR/Messages>.
 
 =head1 METHODS
 
 =head2 new()
 
+    new($name, $content, $language);
+
+This method creates and retusn a message named C<$name> with the content
+C<$content> for the langage of the interface C<$language>.
 
 =head2 getContent()
 
+ getContent($language);
+
+This method returns the content of the message according to the
+language of the interface.
 
 =head2 getName()
     
-
+This method returns the name of the message.
 
 =head1 SEE ALSO
 
@@ -67,7 +81,7 @@ Terminological Resources. In Advances in Natural Language Processing
 
 =head1 AUTHOR
 
-Thierry Hamon <thierry.hamon@lipn.univ-paris13.fr> and Sophie Aubin <sophie.aubin@lipn.univ-paris13.fr>
+Thierry Hamon <thierry.hamon@univ-paris13.fr> and Sophie Aubin <sophie.aubin@lipn.univ-paris13.fr>
 
 =head1 COPYRIGHT AND LICENSE
 
