@@ -115,45 +115,94 @@ __END__
 
 =head1 NAME
 
-Lingua::YaTeA::ParsingPatternRecordSet - Perl extension for ???
+Lingua::YaTeA::ParsingPatternRecordSet - Perl extension for managing the set of the parsing patterns
 
 =head1 SYNOPSIS
 
   use Lingua::YaTeA::ParsingPatternRecordSet;
-  Lingua::YaTeA::ParsingPatternRecordSet->();
+  Lingua::YaTeA::ParsingPatternRecordSet->new($file_path,$tag_set,$message_set,$display_language);
 
 =head1 DESCRIPTION
 
+The module aims at managing the set of parsing pattern records used in
+the term extraction process. Each parsing pattern is associated to a
+record designated by a name defined as the concatenation of the
+Part-of-Speech tags of the parsing pattern. The module provides
+methods for managing the sets of parsing patterns read from a config
+file. The parsing patterns are stored in the field C<PARSING_RECORDS>.
 
 =head1 METHODS
 
 
 =head2 new()
 
+ new($file_path,$tag_set,$message_set,$display_language);
+
+
+The method creates a new parsing pattern set. The parsing patterns are
+read from the config file C<$file_path>. The parameter C<$tag_set>
+provides sets of tags for the candidats, prepositions and determiners
+(this information has been previously loaded thanks to a another
+module). The parameters C<$message_set> and C<display_language> are
+used for printing related information in the right language.
 
 =head2 loadPatterns()
 
+    loadPatterns($file_path,$tag_set,$message_set,$display_language);
+
+The method calls the parser of the file (C<$file_path>) containing the parsing patterns.
+
+creates a new parsing pattern set. The parameter C<$tag_set> provides
+sets of tags for the candidats, prepositions and determiners (this
+information has been previously loaded thanks to a another
+module). The parameters C<$message_set> and C<display_language> are
+used for printing related information in the right language.
 
 =head2 checkContentWords()
 
+    checkContentWords($num_content_words,$num_line);
+
+The method checks if there is at least a content word or a
+part-of-speech tag referring a content word in the pattern, otherwise
+it dies.
 
 =head2 addPattern()
 
+    addPattern($pattern);
+
+The method adds the pattern C<$pattern> in the current record of the
+parsing pattern set.
 
 =head2 getRecord()
 
+    getRecord($name);
+
+The method returns the record of parsing pattern designated by the
+name C<$name> (the concatenation of the Part-of-Speech tags).
 
 =head2 addRecord()
 
+    addRecord($name);
+
+The method creates a new record designated by the
+name C<$name> (the concatenation of the Part-of-Speech tags).
 
 =head2 existRecord()
 
+    existRecord($name);
+
+The method checks if it exists a record designated by the
+name C<$name> (the concatenation of the Part-of-Speech tags).
 
 =head2 getRecordSet()
 
+    getRecordSet();
+
+The method returns the set of parsing pattern records.
 
 =head2 print()
 
+The method prints the set of parsing pattern records.
 
 =head1 SEE ALSO
 
@@ -166,7 +215,7 @@ Terminological Resources. In Advances in Natural Language Processing
 
 =head1 AUTHOR
 
-Thierry Hamon <thierry.hamon@lipn.univ-paris13.fr> and Sophie Aubin <sophie.aubin@lipn.univ-paris13.fr>
+Thierry Hamon <thierry.hamon@univ-paris13.fr> and Sophie Aubin <sophie.aubin@lipn.univ-paris13.fr>
 
 =head1 COPYRIGHT AND LICENSE
 
